<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\ConnectionService;
use Service\DeleteCascadeService;
use Util\Environment;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\Manager;
use Util\Utilities;

class ApiUsageLog extends Crud  {

    use TResponse;

    protected $dataSourceName = 'apiUsageLog';
    /** @var \Service\Manager\Mongo\Manager  */
    protected $organizationMongoManager;
    /** @var Environment */
    protected $environment;

    public function __construct(\Service\Manager\Mongo\Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, Environment $environment) {
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->organizationMongoManager = $organizationMongoManager;
        $this->environment = $environment;
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        /*
            if(empty($sort)){
                $sort = array("CREATED_AT" => -1 );
            }
        */
        $data = $this->mongoManager->findNext($this->dataSourceName, $params);
        $data = Utilities::parseDateTimeGrid($data, ["CREATED_AT"]);

        foreach ($data as &$item) {
            if(!empty($item['ERROR'])){
                if(!empty(\Service\Exception::LABEL_ERROR_API[$item['ERROR']])) {
                    $item['STATUS'] = \Service\Exception::LABEL_ERROR_API[$item['ERROR']];
                } else {
                    $item['STATUS'] = "Erro desconhecido";
                }
                if (!empty($item['PARAM_ERROR'])){
                    foreach ($item['PARAM_ERROR'] as $key => $string){
                        $item['STATUS'] = str_replace("__" . $key . "__", $string, $item['STATUS']);
                    }
                }
            } else if(!empty($item['STATUS_API'])){
                $item['STATUS'] = $item['STATUS_API'];
            }
            if($item['CONCLUDED'] == 'N') {
                if(empty($item['IS_ERROR']) || $item['IS_ERROR'] == 'N') {
                    $item['STOP_READY'] = 'Y';
                }
            }
        }

        $data = Utilities::parseDataSourceZeedhiNext($data);

        // Paginação
        $data["pagination"]["total"] = $this->count($params);
        $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $this->sendResponse($data, 200);
        // Utilities::datasetMongoToZeedhi($response, $dataSet,$this->dataSourceName);

    }

    public function stopApi(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $idLog = $row['_id'];
            /** @var \Service\Api $apiService */
            $apiService = InstanceManager::getInstance()->getService('\Service\Api');
            $apiService->stopApi($this->organizationMongoManager,$this->environment, $idLog);
        } catch(\Exception $ex){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }


    public function count($params) {
	    return $this->mongoManager->countNext($this->dataSourceName, $params);
	}
}