<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DTO;
use Service\PeriodObservation as PeriodObservationService;

class PeriodObservation extends Crud {

    use TResponse;

	protected $dataSourceName = 'periodObservation';
    protected $mongoManager;
    protected $periodObservationService;

    public function __construct(Manager $mongoManager, PeriodObservationService $periodObservationService){
        $this->mongoManager = $mongoManager;
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->periodObservationService = $periodObservationService;
	}

    public function findNext(DTO\Request $request, DTO\Response $response) {
        // Filtragem
        $params = $request->getParameters();
        $observations = $this->periodObservationService->find($this->dataSourceName, $params);
        //  Periodo
        foreach ($observations as &$obs) {
            if (!empty($obs["period"])) {
                $obs["period"]["start"] = $obs["period"]["start"]->format("d/m/Y");
                $obs["period"]["end"] = $obs["period"]["end"]->format("d/m/Y");
            }
        }
        $data = Utilities::parseDataSourceZeedhiNext($observations);
        // Paginação
        $data["pagination"]["total"] = $this->count($params);
        $data["pagination"]["page"] = array_key_exists("page", $params) ? intval($params["page"]) : 1;
        $data["pagination"]["limit"] = array_key_exists("limit", $params) ? intval($params["limit"]) : 9999;
        $this->sendResponse($data, 200);
    }
    
    public function count($params) {
	    //  $filterCriteria = new ZmartFilterCriteria($filter->getDataSourceName(), $filter->getConditions());
	    //  $filterCriteria->setPage(1);
	    //  $filterCriteria->setPageSize(9999);
	    return $this->mongoManager->countNext($this->dataSourceName, $params);
	}
    public function save(DTO\Request $request, DTO\Response $response){
        $row = $request->getDataSet()->getRows()[0];
        $id = isset($row["_id"]) ? $row["_id"] : null;
        $name = $row["name"];
        $period = $row["period"];
        $indicator = isset($row["indicator"]) ? $row["indicator"] : null;
        $dataSource = isset($row["dataSource"]) ? $row["dataSource"] : null;
        $this->periodObservationService->save($id, $name, $period, $indicator, $dataSource);
    }

    public function delete(DTO\Request $request, DTO\Response $response){
        foreach($request->getRow() as $row){
            $this->periodObservationService->delete($row);
        }
    }
}
