<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Crosstab extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_CROSSTAB = "570ea585c0cef5c05f6f5dfd";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getDatasource(){
        return [
            "loadAll" => true,
            "data" => $this->dataSourceAddIndex($this->widgetData),
            "uniqueKey" => self::DATASOURCE_ID_KEY,
            "searchIn" => [
            ],
        ];
    }

    protected function getRows(){
        $cols = [];
        foreach($this->widget["parameters"]["dimensionY"] as $col){
            $cols[] = [
                "name" => $col,
                "label" => $col,
                "conditions" => [
                    "cssClass" => "Bind no frontend  frontend/src/Services/ChartView/Crosstab.ts:getCharts",
                ],
            ];
        }
        return $cols;
    }

    protected function getCell(){
        $widget = $this->widget;
        $cell = [];
        if(!empty($widget["yKeys"])){
            $yKey = $widget["yKeys"][0];
            $column = array_column($widget["columns"], null, "name")[$yKey] ?? null;
            $cell = [
                "name" => $column["name"],
                "reducer" => !empty($column["function"]) ? $column["function"] : "SUM",
                "formatter" => null,
                "reducerInitialValue" => null,
                "conditions" => [
                    "cssClass" => "Bind no frontend frontend/src/Services/ChartView/Crosstab.ts:getCharts",
                ],
            ];
        }
        return $cell;
    }

    public function convertToNext():array {
        $this->fetchWidgetData(); 
        $widget = $this->widget;
        if (!array_key_exists("labelPropertyMaskValue", $widget)) {
            $widget["labelPropertyMaskValue"] = null;
        }
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_CROSSTAB,
            "component" => "ZdCrosstab",
            "chartType" => "ZdCrosstab",
            "cssClass" => "zmartCrosstab",
            "height" =>"100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "cols" => [
                [
                    "name" => $widget["parameters"]["dimensionX"],
                    "label" => $widget["parameters"]["dimensionXName"],
                ]
            ],
            "rows" => $this->getRows(),
            "datasource" => $this->getDatasource(),
            "cell" => $this->getCell(),
            "mask" => $widget["labelPropertyMaskValue"],
            "fixedCols" => true,
            "fixedRows" => true,
        ];
        $title = [
            "name" => "apexChart_" . $widget['id'] . "_title",
            "component" => "ZdText",
            "text" => !empty($widget["label"]) ? $widget["label"] : "",
            "tag" => "h3",
            "cssStyle" => "font-family: Arial Black; font-size: 14px; font-weight: bold;",
        ];
        $style = $this->getViewStyle($this->widget['indicator']);
        $this->widget = $indicatorNext;
        return [
            "name" => $widget['id'],
            "component" => "ZdContainer",
            "ZmartViewStyle" => $style,
            "children" => [$title, $indicatorNext],
            "viewTypeId" => self::TYPE_ID_CROSSTAB,
            "cssClass" => "zmartCrosstabContainer zd-pt-0",
            "height" =>"100%",
            "widget" => $widget,
        ];
    }
    public function getViewStyle($indicator) {
        /** @var \Controller\GridStyle $gridStyleService */
        $gridStyleService = InstanceManager::getInstance()->getService('\Controller\GridStyle');
        return $gridStyleService->getChartViewStyle($indicator);
    }

    public function fetchWidgetData(){
        $widget = $this->widget;
        $postGroupConditions = !empty($widget["postGroupConditions"]) ? $widget["postGroupConditions"] : [];
        $fullAccessProfiles = !empty($widget['parameters']['fullAccessProfiles']) ? $widget['parameters']['fullAccessProfiles'] : [];
        $dimensions = null;

        $limitRows = null;
        /** @var \Service\WidgetData $widgetDataService */
        $widgetDataService = InstanceManager::getInstance()->getService('\Service\WidgetData');
        $this->widgetData = $widgetDataService->fetchWidgetData(
            $widget['_id'], $widget['parameters']['structureId'], null, $widget['parameters']['period'],
            $widget['parameters']['facts'], $dimensions,array(),
            null,$limitRows,$widget['parameters']['drillDownFilters'], false, [],[], $postGroupConditions,
            null,$widget,null,null, $fullAccessProfiles);
    }
}
