<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;
class Grid extends ChartViewAbstract implements ChartViewInterface {

    const TYPE_ID_GRID = "56200912f038efbfa1f75105";

    public $widget;
    public $widgetData;

    function __construct($widget = null) {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getColumnParameters($fields, $columnName, $factsNames) {
        if (!empty($factsNames) && !empty($factsNames[$columnName])) {
            $columnName = $factsNames[$columnName];
        }
        return [$columnName, $fields[array_search($columnName, array_column($fields, "name"))]];
    }

    public function getColumns(){
        $columns = [];
        foreach(array_merge($this->widget["parameters"]["dimensions"], $this->widget["parameters"]["facts"]) as $zmartColumn){
            list($columnName, $columnParameters) = $this->getColumnParameters($this->widget["fields"], $zmartColumn, !empty($this->widget["factsNames"]) ? array_flip($this->widget["factsNames"]) : null);
            $column = [
                "name" => $columnName,
                "label" => !empty($columnParameters["label"]) ? $columnParameters["label"] : $zmartColumn,
                "conditions" => [
                    "cssClass" => "bind no frontend/src/Services/ChartView/Grid.ts:getCharts",
                ],
                "componentProps" => [
                    "component" => "grid_".$this->widget["_id"]."_".$columnParameters["id"],
                ],
            ];
            if (!empty($columnParameters["mask"]["type"])) {
                $column["mask"] = $columnParameters["mask"]["type"];
            }
            if(!empty($columnParameters["type"]) && !empty($columnParameters["expression"]) && $columnParameters["type"] == "fact"){
                $column["aggregation"] = $columnParameters["expression"];
            }
            $columns[] = $column;
        }
        if(!empty($this->widget["zmartTotalized"])){
            $groupedField = $this->getGroupedField($this->widget["fields"]);
            if(!empty($groupedField)){
                $columns[] = [
                    "name" => $groupedField["name"],
                    "grouped" => true
                ];
            }
        }
        return $columns;
    }

    protected function getGroupedField($fields){
        return $fields[array_search(true, array_map(function($field){return !empty($field["grouped"]);}, $fields))];
    }

    public function getDatasource(){
        $datasource = [
            "data" => $this->dataSourceAddIndex($this->widgetData),
            "limit" => empty($this->widget["zmartTotalized"]) ? 25 : count($this->widgetData),
            "uniqueKey" => self::DATASOURCE_ID_KEY,
        ];
        return $datasource;
    }

    public function getViewStyle($indicator) {
        /** @var \Controller\GridStyle $gridStyleService */
        $gridStyleService = InstanceManager::getInstance()->getService('\Controller\GridStyle');
        return $gridStyleService->getChartViewStyle($indicator);
    }

    public function convertToNext():array {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_GRID,
            "component" => "TekGrid",
            "title" => !empty($widget["label"]) ? $widget["label"] : "",
            "showExport" => true,
            "showReload" => false,
            "showFooter" => empty($this->widget["zmartTotalized"]),
            "height" =>"100%",
            "cssStyle" => self::CHART_CSS_STYLE,
            "columnsButton" => true,
            "columns" => $this->getColumns(),
            "datasource" => $this->getDatasource(),
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
