<?php

namespace Service\ChartView;

use Zeedhi\Framework\DependencyInjection\InstanceManager;

class Pie extends ChartViewAbstract implements ChartViewInterface
{

    const TYPE_ID_PIE = "579106aa11e8f41bb00caff7";

    public $widget;
    public $widgetData;

    function __construct($widget = null)
    {
        $this->widget = $widget;
        $this->widgetData = [];
    }

    protected function getXAxis()
    {
        $widget = $this->widget;
        $data = $this->widgetData;
        $categoria = [];
        foreach ($data as $row) {
            $categoria[] = $row[$widget['chartXAxisField']];
        }
        $xAxis = [
            "mask" => !empty($widget['chartXAxisFormat']) ? $widget['chartXAxisFormat'] : null,
            "categories" => $categoria,
            "field" => $widget['chartXAxisField'],
            "labels" => [
                "show" => true,
                "rotate" => !empty($widget['chartXAxisTicksRotation']) ? $widget['chartXAxisTicksRotation'] - 90 : -60,
            ],
            "tickPlacement" => "between"
        ];

        if ($xAxis["labels"]["rotate"] != 0) {
            $xAxis["labels"]["rotateAlways"] = false;
        }

        return $xAxis;
    }

    protected function getYAxis()
    {
        $widget = $this->widget;
        $serieYAxisFormat = !empty($widget['chartSeries'][0]['serieValueFormat']) ? $widget['chartSeries'][0]['serieValueFormat'] : null;
        $yAxis = [
            "mask" => $serieYAxisFormat,
            "dataLabels" => [],
            "labels" => [
                "show" => true,
            ],
            "opposite" => !empty($widget["serieYAxisPosition"]) ? $widget["serieYAxisPosition"] == "right" : false,
        ];

        if (!empty($widget['serieYAxisLabel'])) {
            $yAxis['title'] = [
                "text" => $widget['serieYAxisLabel'],
                "style" => [
                    "fontSize" => "16px"
                ]
            ];
        }

        return $yAxis;
    }

    /* protected function getSeries(){
        $widget = $this->widget;
        $series = [];
        foreach ($this->widgetData as $row) {
            $fact = $widget["serieFieldValue"];
            $series[] = !empty($row[$fact]) ? $row[$fact] : 0;
        }
        return $series;
    } */

    protected function getSeries()
    {
        $widget = $this->widget;
        $widgetDatas = $this->widgetData;
        return  [
            [
                'type' => 'pie',
                'radius' => "50%",
                "center" => ["50%", "50%"],
                'data' => $this->getDataPie(),
                'tooltip' => [
                    'valueFormatter' => null,
                ],
                'emphasis' => [
                    'itemStyle' => [
                        'shadowBlur' => 10,
                        'shadowOffsetX' => 0,
                        'shadowColor' => 'rgba(0, 0, 0, 0.5)'
                    ],
                ],
                'label' => $this->getRotulo(),
                'color' => $this->getSerieColors(),
            ],
        ];
    }

    protected function getRotulo()
    {
        $widget = $this->widget;
        $arrayAux = [];
        $arrayAux['show'] = false;
        if (!empty($widget["serieShowValueLabels"])){
            switch ($widget["serieShowValueLabels"]) {
                case 'value':
                    $arrayAux['show'] = true;
                    $arrayAux['formatter'] = null;
                    return $arrayAux;
                    break;
                case 'label':
                    $arrayAux['show'] = true;
                    $arrayAux['formatter'] = '{b}';
                    return $arrayAux;
                    break;
                case '%':
                    $arrayAux['show'] = true;
                    $arrayAux['formatter'] = '{d}%';
                    return $arrayAux;
                    break;
                case 'false':
                    $arrayAux['show'] = false;
                    break;
            }
        }
        return $arrayAux;
    }

    protected function getDataPie()
    {
        $widget = $this->widget;
        $widgetDatas = $this->widgetData;
        $arrayAux = [];
        foreach ($widgetDatas as $widgetData) {
            $seriesField = $widget["serieFieldValue"];
            if(is_array($seriesField)){
                $seriesField = $seriesField[0];
            }
            $value = $widgetData[$seriesField];
            $name = !empty($widgetData[$widget["chartXAxisField"]]) ? $widgetData[$widget["chartXAxisField"]] : "???";
            $arrayAux[] =
                [
                    'name'  => $name,
                    'value' => $value,
                ];
        }
        if (!empty($widget["serieSort"])){
            if ($widget["serieSort"] == 'label') {
                $arrayAux = $this->ordenarArray($arrayAux, 'name');
            } else if ($widget["serieSort"] == 'value') {
                $arrayAux = $this->ordenarArray($arrayAux, 'value');
            }
        }

        return $arrayAux;
    }

    function ordenarArray($array, $campo) {
        $comparar = function ($a, $b) use ($campo) {
            if ($campo === 'name') {
                return strcmp($a['name'], $b['name']);
            } elseif ($campo === 'value') {
                if ($a['value'] < $b['value']) {
                    return 1;
                } elseif ($a['value'] > $b['value']) {
                    return -1;
                } else {
                    return 0;
                }
            }
            return 0;
        };
        usort($array, $comparar);
        return $array;
    }
    
    protected function getLabels()
    {
        $widget = $this->widget;
        $labels = [];
        foreach ($this->widgetData as $row) {
            $dimension = $widget["serieFieldLabel"];
            $labels[] = !empty($row[$dimension]) ? $row[$dimension] : "";
        }
        return $labels;
    }

    protected function getTooltip()
    {
        $widget = $this->widget;
        return [
            "enabled" => !empty($widget["chartShowTooltip"]) ? $widget["chartShowTooltip"] : false
        ];
    }

    protected function getPlotOptions()
    {
        $widget = $this->widget;
        $pieOptions = [
            "pie" => [
                "labels" => [
                    "show" => true,
                    "name" => [
                        "show" => true,
                    ],
                    "value" => [
                        "show" => true,
                    ],
                ]
            ]
        ];

        return [
            "pie" => $pieOptions
        ];
    }

    protected function getSerieStacked()
    {
        $widget = $this->widget;

        if (!empty($widget['serieStacked'])) {
            return $widget['serieStacked'];
        }
        return false;
    }

    protected function getChart()
    {
        $widget = $this->widget;

        return [
            "type" => "pie",
            "id" => $widget['id'],
            "stacked" => $this->getSerieStacked(),
            "toolbar" => [
                "show" => true,
                "tools" => [
                    "download" => true,
                    "selection" => false,
                    "zoom" => false,
                    "zoomin" => false,
                    "zoomout" => false,
                    "pan" => false,
                    "reset" => false,
                    "customIcons" => []
                ],
            ],
        ];
    }

    protected function getSerieColors()
    {
        $widget = $this->widget;
        if (!empty($widget["chartSeries"][0]["serieColors"])) {
            return array_merge($widget["chartSeries"][0]["serieColors"],self::DEFAULT_COLORS);
        }
        return self::DEFAULT_COLORS;
    }

    protected function getDataLabels()
    {
        $widget = $this->widget;
        $showDataLabel = false;
        $showAsPercent = false;
        if (!empty($widget["serieShowValueLabels"])) {
            $showDataLabel = $widget["serieShowValueLabels"] != "false";
            $showAsPercent = $widget["serieShowValueLabels"] == "%";
        }
        return [
            "enabled" => $showDataLabel,
            "position" => "top",
            "textAnchor" => "middle",
            "showAsPercent" => $showAsPercent,
        ];
    }

    protected function getTitle()
    {
        $widget = $this->widget;
        return [
            "text" => $widget['name'],
            "textStyle" => [
                "fontSize" => "16px",
                "fontFamily" => "Arial Black",
                "fontWeight" => "bold",
            ],
            "align" => "left",
        ];
    }

    protected function getNoData()
    {
        $widget = $this->widget;

        return [
            "text" => !empty($widget["chartMessageNoData"]) ? $widget["chartMessageNoData"] : self::NO_DATA_TEXT,
            "align" => 'center',
            "verticalAlign" => 'middle',
            "offsetX" => 0,
            "offsetY" => 0,
            "style" => [
                "fontSize" => '14px',
            ],
        ];
    }

    protected function getOptions()
    {
        return [
            "legend" => $this->getLegend(),
            "title" => $this->getTitle(),
            "tooltip" => [
                "trigger" => "item",
            ],
            'series' => $this->getSeries(),
        ];
    }

    protected function getLegend()
    {
        $widget = $this->widget;
        $variable = [
            "orient" => "vertical",
        ];
        if (!empty($widget["chartLegendPosition"])){
            $string = $widget["chartLegendPosition"];
            if ($string == "top right") {
                $variable['top'] = "5%";
                $variable['right'] = "5%";
            } else if ($string == "top left") {
                $variable['left'] = "5%";
                $variable['top'] = "5%";
            } else if  ($string == "bottom right") {
                $variable['bottom'] = "0%";
                $variable['right'] = "5%";
            } else if ($string == "bottom left") {
                $variable['left'] = "5%";
                $variable['bottom'] = "0%";
            } else {
                $variable['top'] = "5%";
                $variable['right'] = "5%";
            }

        } else {
            $variable['top'] = "5%";
            $variable['right'] = "5%";
        }
        return $variable;
    }

    public function convertToNext(): array
    {
        $this->fetchWidgetData();
        $widget = $this->widget;
        $widget["pieData"] = $this->widgetData[0];
        if (!array_key_exists('serieValueFormat', $widget)) {
            $widget["serieValueFormat"] = null;
        }
        $indicatorNext = [
            "name" => "apexChart_" . $widget['id'],
            "id" => $widget['id'],
            "widget" => $widget,
            "viewTypeId" => self::TYPE_ID_PIE,
            "component" => "ZdECharts",
            "height" => "100%",
            "fillHeight" => true,
            "cssStyle" => self::CHART_CSS_STYLE,
            "options" => $this->getOptions(),
            "mask" => !empty($widget["serieValueFormat"]) ? $widget["serieValueFormat"] : null,
        ];

        $this->widget = $indicatorNext;
        return $indicatorNext;
    }
}
