import * as esprima from "esprima";

export class ExpressionInterpreter {

	public aggFunctions: any[] = [];
	public functionsExp: any[] = [];
	public logicalExp: any[] = [];

	public functions: any = {
		"SUM": {
			"name": "",
			"function": "SUM",
			"content": "",
			"id": 0
		},
		"AVG": {
			"name": "",
			"function": "AVG",
			"content": "",
			"id": 0
		},
		"MIN": {
			"name": "",
			"function": "MIN",
			"content": "",
			"id": 0
		},
		"MAX": {
			"name": "",
			"function": "MAX",
			"content": "",
			"id": 0
		},
		"COUNT": {
			"name": "",
			"function": "COUNT",
			"content": "",
			"id": 0
		}
	};

	public binaryExpressionObj: any = {
		"firstValue": "",
		"operator": "",
		"secondValue": "",
		"id": 0
	};

	public logicalExpressionObj: any = {
		"operator": "",
		"id": 0
	};

	public static operators: string[] = ["+", "-", "*", "/"];
	public static logicalOperators: string[] = ["==", ">", "<", "!=", ">=", "<="];

	public facts: any[] = [];
	public dimensions: any[] = [];
	public nonFactsColumns: any[] = [];
	public columns: any[] = [];
	public usedColumns: any[] = [];
	public fieldName: string = "";

	public functionToValidateColumn = this.validateAndReturnIsColumn;

	public hasFunctionAdded = false; // Flag: Prevents "function inside a function" error; On finish a function build, resetFlag;
	public hasAggFunctionAdded = false;

	public static readExpression(expressionString: string, paramFacts: any[], paramNonColumnFacts: any[], paramColumnName: string) {
		const interpreter = new ExpressionInterpreter();
		interpreter.facts = paramFacts;
		interpreter.nonFactsColumns = paramNonColumnFacts;
		interpreter.fieldName = paramColumnName;
		const decodedExpression = esprima.parseScript(expressionString);
		const expressionStat: any = decodedExpression.body[0];
		const expression = expressionStat.expression;
		interpreter.buildExpression(expression);
		return {
			"functions": interpreter.aggFunctions,
			"mathExpressions": interpreter.functionsExp
		};
	}

	public static readComparisonExpression(expressionString: string, previousOrPost: any, paramColumns: any, paramUsedColumns: any) {
		const interpreter = new ExpressionInterpreter();
		interpreter.columns = paramColumns;
		interpreter.usedColumns = paramUsedColumns;
		expressionString = expressionString.replace(/ AND | and /g, " && ");
		expressionString = expressionString.replace(/ OR | or /g, " || ");
		if (previousOrPost === "post") {
			interpreter.functionToValidateColumn = interpreter.validateAndReturnIsColumnUsed;
		}

		const decodedExpression = esprima.parseScript(expressionString);
		const expressionStat: any = decodedExpression.body[0];
		const expression = expressionStat.expression;
		return interpreter.buildComparisonExpression(expression);
	}

	public buildExpression(expression: any) {
		let objectToReturn;
		if (expression.type === "BinaryExpression") {
			this.binaryExpressionObj.id += 1;
			const buildedObj = {...this.binaryExpressionObj};

			if (!this.hasFunctionAdded && !this.hasAggFunctionAdded) {
				this.hasAggFunctionAdded = true;
				buildedObj.name = this.fieldName;
				this.functionsExp.push(buildedObj);
			}

			this.getBinaryExpression(buildedObj, expression, this.buildExpression.bind(this), this.validateAndReturnFact.bind(this));
			objectToReturn = buildedObj;
		} else if (expression.type === "CallExpression") {
			if (!this.hasFunctionAdded) {
				objectToReturn = this.getFunctionExpression(expression);
			} else {
				ExpressionInterpreter.throwFunctionInsideFunction();
			}
		} else {
			ExpressionInterpreter.throwUnreconizedExpressionType(expression.type);
		}
		return objectToReturn;
	}

	public buildComparisonExpression(expression: any) {
		let objectToReturn;
		let buildedObj;

		const rightSide = this.getExpressionSide(expression, 'right', this.buildComparisonExpression, this.functionToValidateColumn.bind(this));
		const leftSide = this.getExpressionSide(expression, 'left', this.buildComparisonExpression, this.functionToValidateColumn.bind(this));
		if (expression.type === "LogicalExpression") {
			this.logicalExpressionObj.id += 1;
			buildedObj = {...this.logicalExpressionObj};
			if (expression.operator === "&&") {
				buildedObj.operator = expression.operator;
				buildedObj.firstValue = leftSide;
				buildedObj.secondValue = rightSide;
			} else if (expression.operator === "||") {
				buildedObj.operator = expression.operator;
				buildedObj.expressions = [];
				this.logicalOrConcatOrPush(buildedObj, leftSide);
				this.logicalOrConcatOrPush(buildedObj, rightSide);
			}
			objectToReturn = buildedObj;
		} else if (expression.type === "BinaryExpression") {
			this.binaryExpressionObj.id += 1;
			buildedObj = {...this.binaryExpressionObj};

			if (this.isValidLogicalOperator(expression.operator) || this.isValidOperator(expression.operator)) {
				buildedObj.operator = expression.operator;
			} else {
				ExpressionInterpreter.throwInvalidLogicalOperator(expression.operator);
			}

			buildedObj.firstValue = leftSide;
			buildedObj.secondValue = rightSide;

			objectToReturn = buildedObj;
		} else {
			ExpressionInterpreter.throwShouldBeALogicalExpression();
		}
		return objectToReturn;
	}

	public getBinaryExpression(buildedObj: any, expression: any, functionToDeep: any, functionToValidate: any) {
		buildedObj.operator = this.validateAndReturnOperator(expression.operator);
		buildedObj.firstValue = this.getExpressionSide(expression, 'left', functionToDeep, functionToValidate);
		buildedObj.secondValue = this.getExpressionSide(expression, 'right', functionToDeep, functionToValidate);
	}

	public getFunctionExpression(expression: any) {
		const functionName: string = expression.callee.name.toUpperCase();
		if (this.functions[functionName]) {
			this.hasFunctionAdded = true;
			this.functions[functionName].id += 1;
			const functionToAdd = {...this.functions[functionName]};
			if (!this.hasAggFunctionAdded) {
				functionToAdd.name = this.fieldName;
			} else {
				functionToAdd.name = functionToAdd.function + '_' + this.fieldName + '_' + functionToAdd.id;
			}
			functionToAdd.content = this.getFunctionContent(expression.arguments[0]);
			this.hasFunctionAdded = false;
			this.aggFunctions.push(functionToAdd);
			return functionToAdd;
		}
		ExpressionInterpreter.throwUnmappedFunction(functionName);
		return null;
	}

	public getExpressionSide(expression: any, side: string, functionToDeep: any, functionToValidate: any) {
		let value;
		if (expression[side].type === "BinaryExpression" || expression[side].type === "LogicalExpression") {
			value = functionToDeep(expression[side]);
		} else if (expression[side].type === "CallExpression") {
			value = functionToDeep(expression[side]).name;
		} else if (expression[side].type === "Identifier") {
			value = functionToValidate(expression[side].name);
		} else if (expression[side].type === "Literal") {
			value = expression[side].value;
		} else {
			ExpressionInterpreter.throwUnreconizedExpressionType(expression[side].type);
		}
		return value;
	}

	public getFunctionContent(argument: any) {
		let value;
		if (argument.type === "BinaryExpression") {
			value = this.buildExpression(argument);
		} else if (argument.type === "CallExpression") {
			value = this.buildExpression(argument).name;
		} else if (argument.type === "Identifier") {
			value = this.validateAndReturnFact(argument.name);
		} else if (argument.type === "Literal") {
			value = argument.value;
		}
		return value;
	}

	public logicalOrConcatOrPush(buildedObj: any, side: any) {
		if (side.operator === "||") {
			buildedObj.expressions = buildedObj.expressions.concat(side.expressions);
		} else {
			buildedObj.expressions.push(side);
		}
	}

	public validateAndReturnFact(value: any) {
		if (this.isFact(value)) {
			return value;
		}
		this.throwExpColumnError(value);
		return null;
	}

	public isFact(value: any) {
		return this.facts.includes(value);
	}

	public validateAndReturnOperator(operator: any) {
		if (this.isValidOperator(operator)) {
			return operator;
		}
		ExpressionInterpreter.throwInvalidOperator(operator);
		return null;
	}

	public isValidOperator(operator: any) {
		return ExpressionInterpreter.operators.includes(operator);
	}

	public validateAndReturnIsColumn(column: any) {
		if (this.columns.includes(column)) {
			return column;
		}
		ExpressionInterpreter.throwShouldBeColumn(column);
		return null;
	}

	public validateAndReturnIsColumnUsed(column: any) {
		if (this.usedColumns.includes(column)) {
			return column;
		}
		ExpressionInterpreter.throwShouldBeUsedColumn(column);
		return null;
	}

	public isValidLogicalOperator(operator: any) {
		return ExpressionInterpreter.logicalOperators.includes(operator);
	}

	public throwExpColumnError(column: any) {
		if (this.nonFactsColumns.indexOf(column)) {
			ExpressionInterpreter.throwExpressionWithNonFact(column);
		} else if (this.functions[column]) {
			ExpressionInterpreter.throwFunctionInsideFunction();
		} else {
			ExpressionInterpreter.throwUnreconizedWord(column);
		}
	}

	public static throwUnreconizedWord(word: string) {
		throw new Error("Unreconized Word: " + word);
	}

	public static throwFunctionInsideFunction() {
		throw new Error("Isn't possible build a function inside a function");
	}

	public static throwExpressionWithNonFact(column: string) {
		throw new Error("Column " + column + " should be a fact or a integer. Only operations with facts or integer are allowed.");
	}

	public static throwUnmappedFunction(functionName: string) {
		throw new Error("Unmapped function: " + functionName);
	}

	public static throwUnreconizedExpressionType(expressionType: string) {
		throw new Error("Unreconized Expression Type: " + expressionType);
	}

	public static throwInvalidLogicalOperator(operator: string) {
		throw new Error("Invalid Logical Operator: " + operator);
	}

	public static throwInvalidOperator(operator: string) {
		throw new Error("Invalid Operator: " + operator);
	}

	public static throwShouldBeALogicalExpression() {
		throw new Error("This expression should be a Logical Expression");
	}

	public static throwShouldBeColumn(nonColumn: string) {
		throw new Error("Only columns are allowed to comparise. Non column name: " + nonColumn);
	}

	public static throwShouldBeUsedColumn(nonColumn: string) {
		throw new Error("Only used columns in query are allowed to comparise. Column name: " + nonColumn);
	}
}
