import html2canvas from "html2canvas";
import {Metadata, Http, DateHelper, DatasourceFactory, Utils, Loader} from '@zeedhi/core';
import { Tab, Tabs, Form, Modal, ModalService, Select, DateRange, Dashboard, Text, DialogService, Container, Tooltip, Button, SelectTree, Row } from '@zeedhi/common';
import { DashboardGrid } from '@zeedhi/zd-dashboard-grid';
import { PeriodsService } from '@/Services/PeriodsService';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import { Utilities } from '@/Services/Utilities';
import { ZmartChartsApex } from '@/Services/Chats/ZmartChartsApex';
import { ChartViewFactory } from '@/Services/ChartView/ChartViewFactory';
import { Ordination } from '@/Services/Ordination/Ordination';
import { Filter } from '@/Services/Filter/Filter';
import {IFieldConditionsParam} from "@zeedhi/zd-crosstab-common";
import {AppController} from "@/controllers/AppController";
import { ECharts } from '@zeedhi/zd-echarts';
import { CustomAlertService } from '@/Services/CustomAlertService';

export class DashboardController {
    public tabsHeader: any = '';
    public dashboards: any = '';
    public widget: any = '';
    public countNameFormDashboard: any = 0;
    public nameFormDashboard: any = '';
    public headerTabs: any[] = [];
    public viewTypeId: any;
    public nameTypeChart: any;
    public dashboardCards: any;
    public dashboardId: any;
    public tabIndex: any = '';
    public countSkeleton: any = 0;
    public filterReloadDashboard: any = '';
    public getLastImportationDate: any = '';
    private filterDashboardModal!: Modal;
    periodFilterDashboard: any[] = [];
    private structureLabelFilterDashboard: any = null;
    private structureLabelFilterDashboardName: any = null;
    periodFilter: any[] = [];
    public modalDrilldown!: Modal;
    public modalHiddenIndicators: any = undefined;
    public cardsListBeforeEdit: any = {};
    public hiddenCards: any = [];
    public forceDashboardSave: boolean = false;
    public firstTime: boolean = false;
    private userViewsList: any = [];
    public counter: number = 0;
    public fullscreenCardOn: boolean = false;
    public dashboardOtherFilters: any = {};
    public orgCssStyles: any = {};
    public previousConditions: any = null;
    public postConditions: any = null;
    public timeOutSaveLayout: any = null;
    private dataSourceIds: any = [];

    static readonly LABEL_POSITIONS: any = {
        TOP_LEFT: 0, TOP_RIGHT: 1, BOTTOM_LEFT: 2, BOTTOM_RIGHT: 3
    };

    public async openDrillModal() {
        ZmartChartsApex.isDashboard = false;
        this.modalDrilldown?.destroy();
        this.modalDrilldown = await ModalService.createFromJson("chartView", true);

        const modalName = this.modalDrilldown.name + "_drill_" + Date.now().toString();
        this.modalDrilldown.name = modalName;
        this.modalDrilldown.fullscreen = true;
        if (this.modalDrilldown.children[1] !== undefined) {
            this.modalDrilldown.children[1].children![0].cssStyle = "height:calc(100vh - 64px)";
        }
        this.modalDrilldown.children[0].rightSlot.push({
            "name": modalName + "_closeModalButton",
            "component": "ZdButton",
            "small": true,
            "icon": true,
            "iconName": "close",
            "events": {
                "click": this.closeDrillModal.bind(this, this.modalDrilldown)
            }
        });
        this.modalDrilldown.show();
    }

    public closeDrillModal(modal: Modal) {
        modal.hide();
        localStorage.setItem('drillDownView', "");
        localStorage.setItem('drillDownFilter', "");
        ZmartChartsApex.isDashboard = true;
    }

    public toggleFullScreen(dashName: any) {
        const dashboardGrid = Metadata.getInstance<DashboardGrid>(dashName);
        const slotButtonName = dashName + "_DashboardGridButtonFullscreen";
        const slotButtonNameOff = dashName + "_DashboardGridButtonFullscreenOff";
        const cardId = dashboardGrid.currentCardId;
        const bottomFullScream = Metadata.getInstance<Button>(`${slotButtonName}-${cardId}`);
        const bottomFullScreamOff = Metadata.getInstance<Button>(`${slotButtonNameOff}-${cardId}`);
        if (dashboardGrid.fullscreenCard === null) {
            bottomFullScreamOff.isVisible = true;
            bottomFullScream.isVisible = false;
        } else {
            bottomFullScreamOff.isVisible = false;
            bottomFullScream.isVisible = true;
        }
        if (dashboardGrid.fullscreenCard) {
            dashboardGrid.fullscreenCard = null;
            return;
        }
        if (dashboardGrid.currentCardId !== undefined) {
            dashboardGrid.fullscreenCard = dashboardGrid.currentCardId;
        }
    }
    public async loadTabs() {
        const appController = Loader.getInstance('AppController');
        await appController.loginPromise.then();
        await this.loadTabsAfterLogin();
    }

    public async exportIndicatorImage(dashName: any) {
        const dashboard = Metadata.getInstance<Dashboard>(dashName);
        const currentCard: any = Object.values(this.dashboardCards).find((card: any) => card.id === dashboard.currentCardId);
        const canvas = await html2canvas(document.getElementById(dashboard.currentCardId!)!);
        const image = canvas.toDataURL('image/png');
        Utilities.saveCanvasToPng(image, currentCard.name);
    }

    public changeLabelPosition(dashName: any, newPosition: any) {
        const dashboard = Metadata.getInstance<Dashboard>(dashName);
        const currentCard: any = Object.values(this.dashboardCards).find((card: any) => card.id === dashboard.currentCardId);
        const container = Metadata.getInstance<Container>(currentCard.containerName);
        const options = { ...container.children[0].options };
        switch (newPosition) {
            default:
            case DashboardController.LABEL_POSITIONS.TOP_RIGHT:
                options.legend = {
                    top: "5%",
                    right: "5%",
                    orient: "vertical"
                };
            break;
            case DashboardController.LABEL_POSITIONS.TOP_LEFT:
                options.legend = {
                    top: "5%",
                    left: "5%",
                    orient: "vertical"
                };
            break;
            case DashboardController.LABEL_POSITIONS.BOTTOM_LEFT:
                options.legend = {
                    left: "5%",
                    bottom: "0%",
                    orient: "vertical"
                };
            break;
            case DashboardController.LABEL_POSITIONS.BOTTOM_RIGHT:
                options.legend = {
                    bottom: "0%",
                    right: "5%",
                    orient: "vertical"
                };
            break;
        }
        const component = container.getChildInstance<ECharts>(container.children[0].name);
        component.setOption(options);
    }

    public onMountedLabelPositionDropdown({component}: any) {
        const id = component.name.split("_labelPositionDropDownIcon-")[1];
        const cardElement: any = Object.values(this.dashboardCards).find((card: any) => card._id === id);
        const chartType = ChartViewFactory.getInstanceChartType(cardElement.viewTypeId);
        component.isVisible = chartType.legendPositionDropdownIsVisible;
    }

    public async loadTabsAfterLogin() {
        this.filterReloadDashboard = '';
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        tab.tabs = [];
        const buildMainDashboards = await Filter.filterLoadTabs();
        const loadingRow = Metadata.getInstance<Row>('loadingRowDash');
        this.tabsHeader = buildMainDashboards.data.dataset.dashboards;
        let isVisibleButtom = true;
        let editingMode = true;
        if (Utils.isMobile()) {
            isVisibleButtom = false;
            editingMode = false;
        }
        this.tabsHeader.forEach((tabs: any) => {
            const id = tabs._id;
            const dashName = "dashTab_" + id;
            const dashHeight = (window.innerHeight - 168) + "px";
            const cardFooterSlot = [
                {
                    "name": dashName + "_exportTooltip",
                    "component": "ZdTooltip",
                    "bottom": true,
                    "label": "Salvar como Imagem",
                    "children": [
                        {
                            "name": dashName + "_exportBtn",
                            "component": "ZdButton",
                            "icon": true,
                            "small": true,
                            "iconName": "mdi-file-image",
                            "color": "info",
                            "isVisible": true,
                            "events": {
                              "click": this.exportIndicatorImage.bind(this, dashName),
                            }
                        }
                    ]
                },
                {
                    "name": dashName + "_labelPosition",
                    "component": "ZdTooltip",
                    "bottom": true,
                    "label": "Posição da Legenda",
                    "children": [
                      {
                        "name": dashName + "_labelPositionDropDown",
                        "component": "ZdDropdown",
                        "offsetY": true,
                        "activator": {
                            "name": dashName + "_labelPositionDropDownIcon",
                            "component": "ZdButton",
                            "iconName": "mdi-subtitles-outline",
                            "color": "blue",
                            "icon": true,
                            "isVisible": true,
                            "events": {
                                "onMounted": this.onMountedLabelPositionDropdown.bind(this)
                            }
                        },
                        "children": [
                            {
                                "name": dashName + "_labelPositionDropDown_top",
                                "component": "ZdButton",
                                "flat": true,
                                "label": "Superior Direito",
                                "events": {
                                    "click": this.changeLabelPosition.bind(this, dashName, DashboardController.LABEL_POSITIONS.TOP_RIGHT),
                                }
                            },
                            {
                                "name": dashName + "_labelPositionDropDown_bottom",
                                "component": "ZdButton",
                                "flat": true,
                                "label": "Superior Esquerdo",
                                "events": {
                                    "click": this.changeLabelPosition.bind(this, dashName, DashboardController.LABEL_POSITIONS.TOP_LEFT),
                                }
                            },
                            {
                                "name": dashName + "_labelPositionDropDown_left",
                                "component": "ZdButton",
                                "flat": true,
                                "label": "Inferior Direito",
                                "events": {
                                    "click": this.changeLabelPosition.bind(this, dashName, DashboardController.LABEL_POSITIONS.BOTTOM_RIGHT),
                                }
                            },
                            {
                                "name": dashName + "_labelPositionDropDown_right",
                                "component": "ZdButton",
                                "flat": true,
                                "label": "Inferior Esquerdo",
                                "events": {
                                    "click": this.changeLabelPosition.bind(this, dashName, DashboardController.LABEL_POSITIONS.BOTTOM_LEFT),
                                }
                            }
                        ]
                      }
                    ]
                },
                {
                  "name": dashName + "_cardFooter",
                  "component": "ZdTooltip",
                  "bottom": true,
                  "label": "HIDE",
                  "children": [
                    {
                      "name": dashName + "_hideBtn",
                      "component": "ZdButton",
                      "icon": true,
                      "small": true,
                      "iconName": "mdi-eye-off",
                      "color": "info",
                      "isVisible": isVisibleButtom,
                      "events": {
                        "click": this.hideCard.bind(this, dashName),
                      }
                    }
                  ]
                },
                {
                    "name": dashName + "_DashboardGridTooltipFullscreen",
                    "component": 'ZdTooltip',
                    "bottom": true,
                    "label": 'FULLSCREEN',
                    "children": [
                        {
                            "name": dashName + "_DashboardGridButtonFullscreen",
                            "component": 'ZdButton',
                            "icon": true,
                            "small": true,
                            "iconName": 'mdi-fullscreen',
                            "color": 'secondary',
                            "isVisible": true,
                            "events": {
                                "click": this.toggleFullScreen.bind(this, dashName),
                            },
                        }
                    ],
                },
                 {
                    "name": dashName + "_DashboardGridTooltipFullscreenOff",
                    "component": 'ZdTooltip',
                    "bottom": true,
                    "label": 'Minimizar Tela',
                    "children": [
                        {
                            "name": dashName + "_DashboardGridButtonFullscreenOff",
                            "component": 'ZdButton',
                            "icon": true,
                            "small": true,
                            "iconName": 'mdi-fullscreen-exit',
                            "color": 'secondary',
                            "isVisible": false,
                            "events": {
                                "click": this.toggleFullScreen.bind(this, dashName),
                            },
                        },
                    ],
                },
            ];
            const tabComponent = new Tab(
                {
                    "name": "tab_" + id,
                    "tabTitle": tabs.name,
                    "teste": false,
                    "scrollView": false,
                    "children": [
                        {
                            "name": "tabDashboard_" + id,
                            "component": "ZdContainer",
                            "cssClass": "zd-pa-0",
                            "scrollView": true,
                            "maxHeight": dashHeight,
                            "children": [
                                {
                                    "name": dashName,
                                    "id": id,
                                    "zmartSelected": false,
                                    "zmartSelectedFilter": false,
                                    "component": "ZdDashboardGrid",
                                    "cssClass": "zd-pt-2 zd-pl-1 zd-pr-0 zd-pb-1 zmartDashboard100",
                                    "editingMode": editingMode,
                                    "showGridLines": false,
                                    "removePadding": true,
                                    "isVisible": false,
                                    "allowDragFromIcon": true,
                                    "colNum": 18,
                                    "cards": [],
                                    "events": {
                                        "layoutUpdated": this.dashboardLayoutUpdated.bind(this),
                                    },
                                    "showFullscreen": "bottom",
                                    "showEditHeader": false,
                                    "editHeader": {
                                        "name": dashName + "_invisibleHeader",
                                        "component": "ZdHeader",
                                        "cssStyle": "display: none !important;",
                                    },
                                    "cardFooterSlot": cardFooterSlot
                                }
                            ]
                        },
                        SkeletonMetadata.skeleton(id)
                    ]
                }
            );
            tab.tabs.push(tabComponent);
        });
        tab.activeTab = 0;
        this.tabIndex = tab.activeTab;
        loadingRow.isVisible = false;
         if (tab.activeTab === 0 && this.tabsHeader[0]) {

            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[0].name, this.tabsHeader[0].id);
            this.userViewsList = this.dashboards.data.dataset.userViews;
            this.getTabComponent(tab.tabs[this.tabIndex]).zmartSelected = true; // ZMARTSELECTED como true caso a TAB já tenha sido clicada (Visitada).
            let dashboardIndicators = this.dashboards.data.dataset.mainDashboard.indicators;
            if (!dashboardIndicators) {
                dashboardIndicators = this.dashboards.data.dataset.mainDashboard[0].indicators;
            }
            this.dashboardsTabs(dashboardIndicators, 0);
        }
    }

    private dashboardIsChanged(event: any) {
        if (Object.keys(this.cardsListBeforeEdit).length === 0) {
            return false;
        }
        return event.some((card: any) => this.cardsListBeforeEdit[card.i] && (this.cardsListBeforeEdit[card.i].w !== card.w || this.cardsListBeforeEdit[card.i].h !== card.h || this.cardsListBeforeEdit[card.i].x !== card.x || this.cardsListBeforeEdit[card.i].y !== card.y));
    }

    private getMobileString(): String {
        return Utils.isMobile() ? "_mobile" : "";
    }

    private validadeTimeOutSaveLayout(): Boolean {
        const timeNow = new Date().valueOf();
        let validade = true;
        if (this.timeOutSaveLayout) {
            if (timeNow >= this.timeOutSaveLayout) {
                this.timeOutSaveLayout = timeNow + 1000;
                validade = true;
            }
        } else {
            this.timeOutSaveLayout = timeNow + 1000;
            validade = true;
        }
        return validade;
    }

    public async dashboardLayoutSave(event: any) {
        if ((this.dashboardIsChanged(event) || this.forceDashboardSave) && this.validadeTimeOutSaveLayout()) {
            if (!this.firstTime) {
                this.forceDashboardSave = false;
                event = event.map(function (card: any) {
                    if (card.hidden) {
                        delete card.hidden;
                    }
                    return card;
                });
                if (this.hiddenCards) {
                    const hiddenCardsList = this.hiddenCards.map(function (card: any) {
                        const newCard = card;
                        card.hidden = true;
                        return newCard;
                    });
                    event = [...event, ...hiddenCardsList];
                }
                const data = {
                    'dataset': [
                        {
                            "viewId": this.tabsHeader[this.tabIndex]._id + this.getMobileString(),
                            "data": event,
                            "isMobile": Utils.isMobile()
                        }
                    ],
                    "disableLoader": false,
                    "requestType": 'DataSet'
                };
                try {
                    this.dashboardSetSavingText(true);
                    await Http.post('/view/saveUserView', data);
                    this.dashboardSetSavingText(false);
                } catch (err) {
                    console.error(err);
                } finally {
                    this.dashboardSetSavingText(false);
                }
            } else {
                this.firstTime = false;
            }
        }
    }

    public dashboardLayoutUpdated({ event }: any) {
        this.dashboardLayoutSave(event);
    }

    private dashboardSetSavingText(saving: boolean) {
        const text = Metadata.getInstance<Text>("dashboardSavingText");
        text.isVisible = saving;
    }

    public async closeModalVisibilityCard() {
        this.modalHiddenIndicators.hide();
    }

    private async loadHiddenIndicatorsInDatasource() {
        this.hiddenCards = await Http.get("/view/getHiddenIndicators");
        this.hiddenCards = this.hiddenCards.data;
        this.hiddenCards.forEach(function (card:any) {
            card.i = card._id;
        });
    }

    public resetUserViewsConfirmationDialog() {
        DialogService.show({
            name: 'buttons-dialog',
            type: 'warning',
            title: 'Restaurar padrão de visualização',
            text: 'Deseja realmente retornar para a visão original?',
            buttons: [
              { name: 'no', component: 'ZdButton', label: 'NO', events: this.resetUserViewsCancel, outline: true},
              { name: 'yes', component: 'ZdButton', label: 'YES', events: this.resetUserViewsConfirm},
            ],
          });
    }

    private resetUserViewsCancel: any = {
        click: () => DialogService.hide(),
    };

    private resetUserViewsConfirm: any = {
        click: () => this.resetUserViews(),
    };

    public async resetUserViews() {
        DialogService.hide();
        const data = {
            'dataset': [
                {
                    "viewId": this.tabsHeader[this.tabIndex]._id + this.getMobileString(),
                    "isMobile": Utils.isMobile()
                }
            ],
            "disableLoader": false,
            "requestType": 'DataSet'
        };
        await Http.post('/view/resetUserViews', data);
        this.reload();
    }

    public async openModalVisibilityCard() {
        await this.loadHiddenIndicatorsInDatasource();
        if (!this.modalHiddenIndicators) {
            this.modalHiddenIndicators = this.createIndicatorVisibilityModal();
        }
        this.reloadHiddenIndicatorsDatasource();
        this.modalHiddenIndicators.show();
    }

    public async showHiddenCard({ row, component }: any) {
        component.disabled = true;
        const rowId = row._id ? row._id : null;
        if (rowId) {
            const indicator = this.getIndicator(row, 1, 1);
            this.dashboardCards["override_" + rowId + this.counter] = row;
            this.hiddenCards = this.hiddenCards.filter((obj: any) => obj._id !== row._id);
            const tabsDashboard = Metadata.getInstance<Tabs>("tabsDashboard");
            this.getTabComponent(tabsDashboard.tabs[this.tabIndex]).cards.push(indicator);
            this.reloadHiddenIndicatorsDatasource();
            this.forceDashboardSave = true;
        }
    }

    public hideCard(dashName: any) {
        const dashboard = Metadata.getInstance<Dashboard>(dashName);
        const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
        const index = this.getTabComponent(tabName.tabs[this.tabIndex]).cards.findIndex((obj: { i: any }) => obj.i === dashboard.currentCardId);
        if (index !== -1) {
            const removedCard = this.getTabComponent(tabName.tabs[this.tabIndex]).cards.splice(index, 1);
            this.hiddenCards.push(removedCard[0]);
        }
    }

    private reloadHiddenIndicatorsDatasource() {
        this.modalHiddenIndicators.children[1].datasource.updateData(this.hiddenCards);
        if (this.modalHiddenIndicators.childrenInstances && this.modalHiddenIndicators.childrenInstances.length && this.modalHiddenIndicators.childrenInstances.length > 0) {
            this.modalHiddenIndicators.childrenInstances[1].datasource.updateData(this.hiddenCards);
            this.modalHiddenIndicators.childrenInstances[1].reload();
        }
    }

    private createIndicatorVisibilityModal() {
        return ModalService.create({
            name: 'visibilityModal_',
            persistent: true,
            children: [
                {
                    name: "listCardVisibility_name",
                    component: "ZdText",
                    text: "Lista de cards não visíveis",
                    tag: "h3",
                },
                {
                    name: 'listCardVisibility_',
                    component: 'TekGrid',
                    cssClass: 'zd-my-4',
                    showFooter: false,
                    showLayoutOptions: false,
                    showReload: false,
                    showSearch: true,
                    showSelectAllGrid: false,
                    showSummaryTotal: false,
                    dragColumns: false,
                    resizeColumns: false,
                    toolbarSlot: [],
                    columns: [
                        {
                            name: 'label',
                            label: 'Nome',
                            sortable: false,
                        },
                        {
                            name: 'toggleVisibilityAction_',
                            type: 'action',
                            sortable: false,
                            width: "16px",
                            children: [
                                {
                                    name: 'toggleVisibilityBtn_',
                                    component: 'ZdButton',
                                    disabled: false,
                                    icon: true,
                                    small: true,
                                    color: 'primary',
                                    iconName: 'eye',
                                    events: {
                                        click: this.showHiddenCard.bind(this),
                                    },
                                },
                            ],
                        },
                    ],
                    datasource: DatasourceFactory.factory({
                        uniqueKey: 'i',
                        type: "tek-memory",
                        data: this.hiddenCards,
                    }),
                },
                {
                    name: 'visibilityCardFooter_',
                    component: 'ZdFooter',
                    color: 'transparent',
                    padless: true,
                    rightSlot: [
                        {
                            name: 'visibilityCancelButton_',
                            component: 'ZdButton',
                            cssClass: 'zd-mr-2',
                            outline: true,
                            label: 'CLOSE',
                            events: {
                                click: this.closeModalVisibilityCard.bind(this),
                            },
                        },
                    ],
                },
            ],
        });
    }

    private getTabComponent(tabs:any) {
        return tabs.children[0].children[0];
    }

    public async clickTab({ tabIndex }: any) {
        this.tabIndex = tabIndex;
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        const tabComponent = this.getTabComponent(tab.tabs[tabIndex]);
        if (tabComponent.zmartSelected === false) {

            this.dashboards = "";
            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[tabIndex].name, tabComponent.id);
            this.userViewsList = this.dashboards.data.dataset.userViews;

            if (this.periodFilterDashboard[0] !== undefined) {
                this.getTabComponent(tab.tabs[tabIndex]).cards = [];
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex, true);
            } else {
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex);
            }
        }
        this.getTabComponent(tab.tabs[tabIndex]).zmartSelected = true;
        this.getTabComponent(tab.tabs[tabIndex]).zmartSelectedFilter = true;
    }

    public styleToRemoveHtml(element: any, iterator: any) {
        document.getElementsByTagName('head')[0].removeChild(element);
    }

    public removeAppliedCustomStyles() {
        const styles = document.querySelectorAll('[id^="Zmart_custom_"]');
        Array.prototype.forEach.call(styles, this.styleToRemoveHtml);
    }

    public async dashboardsTabs(dashboards: any, tabIndex: any, filter: boolean = false) {
        this.hiddenCards = [];
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        const tabComponent = this.getTabComponent(tab.tabs[this.tabIndex]);
        if (tabComponent.zmartSelectedFilter === false) {
            const dashboardIndicatorsIds: any[] = [];
            const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            this.getTabComponent(tabName.tabs[tabIndex]).cards = [];

            if (this.filterReloadDashboard !== '') {
                dashboards.forEach(async (dashboard: any) => {
                    dashboard.parameters.period.start = this.periodFilterDashboard[0];
                    dashboard.parameters.period.end = this.periodFilterDashboard[1];
                });
            }

            dashboards.forEach(function (dashboard: any) {
                if (!dashboardIndicatorsIds.includes(dashboard.indicator)) {
                    dashboardIndicatorsIds.push(dashboard.indicator);
                }
            });

            if (dashboards[0] !== undefined) {
                this.getLastImportationDate = "";
                this.getLastImportationDate = await Filter.filterGetLastImportationDates(dashboardIndicatorsIds, this.tabsHeader[tabIndex].name);
            }

            Ordination.sort(dashboards); // Ordenando os indicadores
            this.indicatorProcess(dashboards, tabIndex);
        }

        if (filter === true) {
            this.getTabComponent(tab.tabs[tabIndex]).zmartSelected = true;
            this.getTabComponent(tab.tabs[tabIndex]).zmartSelectedFilter = true;
        }

    }

    private getIndicator(dashboard: any, defaultX: any, defaultY: any) {
        let indicatorPadding = "zd-pa-2";
        if (dashboard.viewType === "dashboardCard") {
            indicatorPadding = "dashboardCard zd-pa-0";
        }
        return {
            "i": dashboard._id,
            "path": "dashboard/dashboards",
            "local": true,
            "cssClass": indicatorPadding,
            "overrideNamedProps": {
                "NAME": "override_" + dashboard._id + this.counter,
            },
            "name": dashboard.name,
            "h": dashboard.height ? Math.floor(dashboard.height) : 3,
            "w": dashboard.width ? Math.floor(dashboard.width) : 3,
            "x": (dashboard.x === undefined) ? defaultX : dashboard.x,
            "y": (dashboard.y === undefined) ? defaultY : dashboard.y,
        };
    }

    private indicatorProcess(dashboards: any, tabIndex: any) {
        let widthCounter = 0;
        let heightCounter = 0;
        let biggestHeight = 0;
        let userViews: any = null;
        const indicators: any[] = [];
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        const tabComponent = this.getTabComponent(tab.tabs[this.tabIndex]);
        const idUserViews = tabComponent.id + this.getMobileString();
        if (this.userViewsList && this.userViewsList[idUserViews]) {
            userViews = this.userViewsList[idUserViews];
        }
        this.viewTypeId = {};
        this.dashboardCards = {};
        let addToIndicator = true;
        let currentX = 0;
        const currentY = 0;
        dashboards.forEach(async (dashboard: any) => {
            dashboard.width = Math.floor(dashboard.width);
            dashboard.height = Math.floor(dashboard.height);
            addToIndicator = true;
            if (widthCounter + dashboard.width > 18) {
                heightCounter += biggestHeight;
                widthCounter = 0;
                biggestHeight = 0;
            }
            this.dashboardCards["override_" + dashboard._id + this.counter] = dashboard;
            if (!dashboard.width) {
                dashboard.width = 3;
            }
            if (!dashboard.height) {
                dashboard.height = 3;
            }
            const isUndefinedXY = (dashboard.x === undefined && dashboard.y === undefined);
            if (Utils.isMobile()) {
                dashboard.width = 18;
            }
            const indicator = this.getIndicator(dashboard, widthCounter, heightCounter);
            if (userViews) {
                if (userViews[dashboard._id]) {
                    if (Utils.isMobile()) {
                        userViews[dashboard._id].w = 18;
                    }
                    indicator.h = userViews[dashboard._id].h;
                    indicator.w = userViews[dashboard._id].w;
                    indicator.x = userViews[dashboard._id].x;
                    indicator.y = userViews[dashboard._id].y;
                } else if (userViews.hidden[dashboard._id]) {
                    if (Utils.isMobile()) {
                        userViews.hidden[dashboard._id].w = 18;
                    }
                    indicator.h = userViews.hidden[dashboard._id].h;
                    indicator.w = userViews.hidden[dashboard._id].w;
                    indicator.x = userViews.hidden[dashboard._id].x;
                    indicator.y = userViews.hidden[dashboard._id].y;
                    addToIndicator = false;
                } else if (isUndefinedXY) {
                    const reducedIndicator = this.reduceHeightWidth(dashboard);
                    const arrangedIndicator = this.arrangeIndicator(reducedIndicator, currentX, currentY);
                    currentX = arrangedIndicator.x + arrangedIndicator.width;
                    const { height, width, x, y } = arrangedIndicator;
                    indicator.h = height;
                    indicator.w = width;
                    indicator.x = x;
                    indicator.y = y;
                    this.firstTime = true;
                }
            } else if (userViews !== null) {
                const reducedIndicator = this.reduceHeightWidth(dashboard);
                const arrangedIndicator = this.arrangeIndicator(reducedIndicator, currentX, currentY);
                currentX = arrangedIndicator.x + arrangedIndicator.width;
                const { height, width, x, y } = arrangedIndicator;
                indicator.h = height;
                indicator.w = width;
                indicator.x = x;
                indicator.y = y;
                this.firstTime = true;
            }
            dashboard.h = indicator.h;
            dashboard.w = indicator.w;
            if (addToIndicator) {
                indicators.push(indicator);
                widthCounter += dashboard.width;
                if (biggestHeight === 0 || biggestHeight < dashboard.height) {
                    biggestHeight = dashboard.height;
                }
            } else {
                this.hiddenCards.push(indicator);
            }
        });
        this.cardsListBeforeEdit = this.getCardsListById(indicators);
        this.getTabComponent(tab.tabs[tabIndex]).cards = indicators;
    }

    // Função para reduzir a altura e largura em 30%
    public reduceHeightWidth(indicator: any) {
        const newH = Math.round(indicator.height * 0.7);
        const newW = Math.round(indicator.width);
        return { ...indicator, height: newH, width: newW };
    }

    public arrangeIndicator(indicator: any, currentX: number, currentY: number) {
        // Verifique se o gráfico cabe na linha atual
        if (currentX + indicator.width > 18) {
          currentX = 0; // Reinicie x quando uma nova linha começar
          currentY += indicator.height; // Avance para a próxima linha
        }
        // Defina as posições x e y para o gráfico
        const newX = currentX;
        const newY = currentY;
        // Atualize a posição x para o próximo gráfico na mesma linha
        currentX += indicator.width;
        return {
          ...indicator,
          x: newX,
          y: newY,
        };
      }

    public calculoDashboardY(indicator: any) {
        if (indicator.w !== 18) {
            return indicator.y + 6;
        }
        return indicator.y;
    }

    public calculoDashboardX(indicator: any) {
        if (indicator.w !== 18) {
            return indicator.x + 6;
        }
        return indicator.x;
    }

    private getCardsListById(indicators: any) {
        return indicators.reduce((obj: any, item: any) => {
            obj[item.i] = {
                "h": item.h,
                "w": item.w,
                "x": item.x,
                "y": item.y,
            };
            return obj;
          }, {});
    }

    private setCardBackgroundColor(color: any, id: any) {
        const element = document.querySelectorAll<any>("[i='" + id + "']")[0];
        if (element) {
            element.style["background-color"] = color;
        }
    }

    public async onMountedChildrenDashboard({component}: any) {
        if (this.dashboardCards) {
            if (this.dashboardCards[component.name]) {
                this.dashboardCards[component.name].containerName = component.name;
            }
            const dashboard = this.dashboardCards[component.name];
            const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            let period = "";
            if (dashboard && dashboard.parameters && dashboard.parameters.period) {
                period = dashboard.parameters.period;
            }
            let structureId = "";
            let structureName = "";
            if (this.structureLabelFilterDashboard !== null) {
                structureId = this.structureLabelFilterDashboard;
                structureName = this.structureLabelFilterDashboardName;
            }
            this.buildSkeletonChart(component);
            component.children[0].height = this.heightCard(dashboard.height - 1) + "px";
            component.children[0].fillHeight = true;
            this.getTabComponent(tabName.tabs[this.tabIndex]).isVisible = true;
            tabName.tabs[this.tabIndex].children[1].isVisible = false;
            const tabComponent = this.getTabComponent(tabName.tabs[this.tabIndex]);
            const buildWidget = await Filter.filterBuildWidgetByViewIdNext(period, dashboard, structureId, tabComponent.id, structureName);
            this.widget = '';
            if (buildWidget.data.dataset) {
                if (buildWidget.data.dataset.error && buildWidget.data.dataset.error.length > 0) {
                    Utilities.widgetErrorMessageAlerts(buildWidget.data.dataset.error);
                }
                this.widget = buildWidget.data.dataset.widget;
                this.viewTypeId[this.widget[0].id] = dashboard.viewTypeId;
                const charts = ZmartChartsApex.build(this.widget, null, component, this);
                const chartType = ChartViewFactory.getInstanceChartType(this.viewTypeId[this.widget[0].id]);
                this.widget = chartType.getWidgetDashboard(this.widget, dashboard);
                const widgetCharts = charts.getCharts();
                widgetCharts[0].grid = {
                    "sm": "12",
                    "lg": "12",
                    "cols": "18",
                    "md": "12",
                };
                widgetCharts[0].height = this.heightCard(dashboard.height - 1) + "px";
                widgetCharts[0].fillHeight = true;
                if (widgetCharts[0].component !== "ZdCard") {
                    const componentDataImportation = this.periodLabel(dashboard);
                    widgetCharts.push(componentDataImportation); // Setando na última posição do Json
                    if (widgetCharts[0].bgColor) {
                        this.setCardBackgroundColor(widgetCharts[0].bgColor, widgetCharts[0].id);
                    }
                }
                if (this.widget[0].name === 'notFound') {
                    component.children = this.widget;
                } else {
                    component.children = widgetCharts;
                }
            }
        }
    }

    public buildSkeletonChart(component: any) {
        component.children = [
            SkeletonMetadata.skeletonIndicator().children[0]
        ];
    }

    public periodLabel(dashboard : any) {
        let componentDataImportation: any = {};
        const updateDate = this.getLastImportationDate.data.dataset.lastImportDates[dashboard.dataSourceId];
        if (dashboard.parameters.period) {
            let componentText: string = "";

            if (dashboard.parameters.period.periodLabel === undefined) {
                componentText = dashboard.parameters.period.start + " - " + dashboard.parameters.period.end;
            } else {
                componentText = dashboard.parameters.period.periodLabel;
            }

            if (updateDate) {
                componentText += " * Atualizado em : " + updateDate;
            }
            const dashId = dashboard.id;
            const indicator = dashboard.indicator;
            componentDataImportation = {
                "name": dashId + "_callDescriptionValue_container" + this.counter,
                "component": "ZdContainer",
                "cssClass": "zd-pa-0 zd-pt-2 zd-pl-3",
                "width": "100%",
                "grid": {
                    "sm": "12",
                    "lg": "12",
                    "cols": "18",
                    "md": "12",
                },
                "children": [
                    {
                        "name": dashId + "_callDescriptionValue" + this.counter,
                        "component": "ZdText",
                        "width": "100%",
                        "cssClass": "zd-text-left",
                        "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                        "text": componentText,
                    }
                ]
            };
        }
        this.counter += 1;
        return componentDataImportation;
    }

    public heightCard(height: any) {
        return height * 130;
    }

    public instanceTab() {
        const tabs = Metadata.getInstance<Tabs>('tabsDashboard');

        tabs.tabs.forEach(function (tab: any) {
            tab.children[0].children[0].zmartSelectedFilter = false;
            tab.children[0].children[0].zmartSelected = false;
            tab.children[0].children[0].isVisible = false;
            tab.children[1].isVisible = true;
        });
    }

    public indicatorFilterDashboard(filters: any, actionOnClick: any) {
        if (actionOnClick === "filterWithPeriod" && filters.period) {
            const dateStart = filters.period.start;
            const dateEnd = filters.period.end;
            this.periodFilterDashboard = [dateStart, dateEnd];
            this.periodFilter = [dateStart, dateEnd];
        }
        delete filters.period;
        this.reload(filters);
    }

    public finishImportation(data: any) {
       console.log('finishImportation');
    }

    public async reloadButton() {
        await this.reload();
    }

    public async reload(otherFilters?: any) {
        this.instanceTab();
        const dashboards = await Filter.dashboardFilter(this.tabsHeader[this.tabIndex].name, this.tabsHeader[this.tabIndex].id);
        const dashboardsIndicators = dashboards.data.dataset.mainDashboard.indicators;
        this.userViewsList = dashboards.data.dataset.userViews;
        this.filterReloadDashboard = '';
        if (this.periodFilterDashboard[0] !== undefined) {
            this.filterReloadDashboard = dashboardsIndicators;
        }
        this.dataSourceIds = [];
        dashboardsIndicators.forEach((dashboard: any) => {
            this.dataSourceIds.push(dashboard.dataSourceId);
        });

        if (otherFilters) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.drillDownFilters = otherFilters;
            });
        }

        if (this.previousConditions) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.previousConditions = this.previousConditions;
            });
            this.previousConditions = null;
        }

        if (this.postConditions) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.postConditions = this.postConditions;
            });
            this.postConditions = null;
        }

        if (this.periodFilterDashboard[0] !== undefined) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.period = {
                    "start": this.periodFilterDashboard[0],
                    "end": this.periodFilterDashboard[1],
                };
            });
            this.filterReloadDashboard = dashboardsIndicators;
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex, true);
        } else {
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex);
        }
    }

    public filterDashboard() {
        this.createFilterDashboardModal();
    }

    public async createFilterDashboardModal() {
        this.filterDashboardModal = await ModalService.createFromJson('dashboard/modal/filterDashboard', true);
        this.filterDashboardModal.show();
    }

    public closeFiterDashboardModal() {
        this.filterDashboardModal.hide();
        this.filterDashboardModal?.destroy();
    }

    public async onMountedFilterDashboard() {
        // Se formulário não foi montado, não faz nada
        if (Metadata.getInstances('structureTypeForm').length === 0 || !Metadata.getInstances('structureTypeForm')[0]) {
            return;
        }
        const form = Metadata.getInstance<Form>('structureTypeForm');
        const structureLabel = Metadata.getInstance<Select>('structure_label');
        const period = form.getChildInstance<DateRange>('period');

        const periodDropdown = period.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;

        this.generatePeriodHelperOptions(period);
        const data = await Filter.filterGetStructureByUser();
        structureLabel.datasource.updateData(data.data.dataset.getStructureByUser);
        if (this.structureLabelFilterDashboard !== null) {
            structureLabel.setValue(this.structureLabelFilterDashboard);
        }

        if (this.periodFilterDashboard[0] !== undefined) {
            form.value.period = this.periodFilter;
        }
    }

    public async generatePeriodHelperOptions(period: any) {
        const periodDate = await Http.post("/periods");
        const data = periodDate.data.data;
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode));
        }
        data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
            const service = new PeriodsService();
            const periods = service.decodePeriod(option.periodsCode);
            return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
        }));
        period.helperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        period.helperValue = data.periodsCode;
        period.updateHelperHint();
    }

    public async saveFiterDashboard() {
        this.structureLabelFilterDashboard = null;
        const form = Metadata.getInstance<Form>('structureTypeForm');
        let dataForm = false;
        if (!!form.value.period && !!form.value.period.length) {
            const dateStart = form.value.period[0];
            const dateEnd = form.value.period[1];
            dataForm = true;
            this.periodFilterDashboard = [dateStart, dateEnd];
            this.periodFilter = [dateStart, dateEnd];
        }
        if (dataForm === false) {
            this.periodFilterDashboard = [];
        }

        if (form.value.structure_label !== undefined && form.value.structure_label !== null) {
            this.structureLabelFilterDashboard = form.value.structure_label;
            const selectComponent = Metadata.getInstance<SelectTree>("structure_label");
            if (selectComponent) {
                const structureRow: any = selectComponent?.datasource?.data.find((data: any) => data._id === this.structureLabelFilterDashboard);
                this.structureLabelFilterDashboardName = structureRow.name;
            }
        }
        this.closeFiterDashboardModal();
        await this.reload();
    }

    public get isVisibleMobile() {
        if (Utils.isMobile()) {
            return false;
        }
        return true;
    }
}
